# Imports
import json # to work with .json files  (.json-->.csv)
import pandas as pd # to convert files
import wget # to download the data at links
import gzip # (json.gz-->.json) ^ (.json-->.csv)
import shutil # (json.gz-->.json) ^ (.json-->.csv)
import os # to extract and convert files from the dir
import urllib.request # to check the size of the file
import sys
import time


def get_links(file_name):
    """
    A function to take the index.json file, extract all the location(links)
    then filter out the donwloadable .json.gz links and store them in a list then write them to a .txt file
    At the end call the download function with the filtered list of links
    :param file_name: json file
    :return: run download function to get the files at the links
    """
    print(f"Entering get_links({file_name}) function\n")
    try:
        index_file = pd.read_json(file_name)
    except:
        pass
    # Getting all the links from all the data items to a list
    links_to_download = []
    for data in range(len(index_file['reporting_structure'])):
        for link in range(len(index_file['reporting_structure'][data]['in_network_files'])):
            links_to_download.append(index_file['reporting_structure'][data]['in_network_files'][link]['location'])
    print(f"Total {len(links_to_download)} links were found in {file_name}")

    links_to_download=list(set(links_to_download))

    # writing the links to a .txt file
    with open(file_name[:-4]+"txt", 'w') as f:
        for i in links_to_download:
            f.write(i + '\n')
            print(f"link {i} are written to {file_name[:-4]}txt")
    print(f"{len(links_to_download)} links have been written to file {file_name[:-4]}txt")
    print(f"\n\nDone with get_links() function and moving to download() function")
    return download(links_to_download)


def download(links_to_download):
    """
    A function which takes the link from the list, go the url
    check the file size,
    if the size is allowed to be downloaded then downloads it
    else ignores it.

    :param links_to_download: a list with .json.gz links
    :return: run the unzip_extract_convert function
    """
    print(f"\nEentering download function with download({links_to_download})\n")
    # Downloading the files at the given links using wget command
    for url in links_to_download:
        #url=links_to_download[25]
        try:
            # checking for larger files to ignore
            file = urllib.request.urlopen(url)
            print(f"Getting data at {url} having {file.length / 1e6} MBs\n")
            if file.length < 2e8:  # only checking for 200 MBs 2e+8 bytes #TODO: Set the file size
                wget.download(url,out='./json_files/')
                print(f"\nDOWNLOADED\n{url}\n")
                print(f"\nDone with download() and moving to unzip_extract_convert()\n")
                unzip_extract()
                print("One file done\nEXTING...")
                #break # checking for one file
            else:
                print(f"The file size is {file.length / 1e6} MBs which is greater then 200 MBs... IGNORING\n")
        except:
            print("Invalid Link. No Downloadble File  found")


def unzip_extract():
    """
    A function which checks all the .json.gz downloaded files in the current dir.
    if the extension matches (.json.gz) extracts them with the same name .json.
    :return:
    """
    print(f"Entering unzip_extract() function\n")
   # Unzip the .json.gz
    path_of_the_directory = './json_files/'
    ext = ('.gz')
    for files in os.listdir(path_of_the_directory):
        if files.endswith(ext):
            name = files[2:-3]
            print("----------------------------------WORKING---------------------------------------")
            # Extracting json.gz to .json and then converting .json to .csv and saving with the name
            with gzip.open("./json_files/"+files, 'rb') as f_in:
                print("EXTRACTING...")
                with open("./json_files/"+name, 'wb') as f_out:
                    shutil.copyfileobj(f_in, f_out)
                print(f"\nDone with EXTRACTING the file \n{files} \n \t\t\t and is saved as \n{name}\n")
                print(f"DELETING the extracted file...") 
                os.remove("./json_files/"+files)
                print("Done deleting the file")
                             

        else:
            if files.endswith('.json') or files.endswith('.txt') or files.endswith('.py') or files=="json_files":
                print(f"file {files} is in .json, .py, .txt or a dir (json_files) NOT EXTRACTABLE..")
            else:
                print(f"Deleting useless files {files} DELETED...")
                os.remove("./json_files/"+files)
                print("Done deleting the file")
            
    print(f"Done with unzip_extract() function")           







if __name__ == "__main__":
    #read the path
    file_path = "./"
    #list all the files from the directory
    file_list = os.listdir(file_path)
    # print(file_list)
    ext='_index.json'
    for file in file_list:
        file_name = file
        if file.endswith(ext):
            print('-----------------------------------------')
            print(f"STARTING THE SCRIPT FOR FILE {file_name}")
            get_links(file_name)
            print('-----------------------------------------')
            print(f"FILE {file_name} done\n NEXT...")
            os.remove(file_name)
        else:
            print(f"File {file_name} not _index.json")
        
################################################################################
###################################Old##########################################
#################################################################################
    # print(f"Run from command line as:\n"
    #       f"python3 main.py 'filename'")
    # if sys.argv[1] and sys.argv[1][-5:]==".json":
    #     file_name=sys.argv[1]
    #     get_links(sys.argv[1])
    # else:
    #     print("WRONG inputs")
    #     print("Try using python3 main.py 'filename'\nYour file extension should be .json and the file should be in the same directory")
