import argparse

import pandas as pd
from sklearn.preprocessing import LabelEncoder
import os

def get_all_csvs(config):
    """
    A simple function to get a list of all the csv files in the given directory, by default it is the current dir.
    Pass that list to merge all the csvs based on defined columns
    :param file_path: directory of cvs
    :return: pass the list of csvs to merge_all_csvs() function
    """
    files = []
    for file in os.listdir(config.file_path):
        if file.endswith(".csv"):
            files.append(file)
        else:
            print(f"File --> {file} is not .csv")
    print(f"Moving to merge_all_csv() with the list of files\nTotal files to be merged = {len(files)}")
    return merge_all_csvs_encode(files, config.output_file)

def merge_all_csvs_encode(list_files,output_file):
    """
    A function to merge the csvs files given as list and output the final file as csv
    :param list_files: list of paths to csv files
    :param output_file: name for the final csv
    :return: saves a .csv with output_file as name
    """
    done=0
    failed=0
    final_df = pd.DataFrame(columns=['reporting_entity_type', 'billing_code_types', 'billing_code', 'negotaited_rate', 'negotiated_type','tin_value'])
    for f in list_files:
        df=pd.read_csv(f,sep=";")
        if 'negotaited_rate' not in df.columns:
            failed += 1
            print(f"file --> {f} does not have negotiated column. So not merged")
            continue
        try:
            final_df=pd.concat([final_df,df],join="inner")
            final_df = final_df.loc[:,final_df.columns.isin(['reporting_entity_type', 'billing_code_types', 'billing_code','negotaited_rate', 'negotiated_type','tin_value'])]
            final_df.drop_duplicates(inplace=True)
            done+=1
            #os.remove(f)
        except:
            failed+=1
            print(f"Failed to merged the file-->{f}.\nNot Deleted. Check for manual merging..")

    # some csv websites do not provide state values... Comment it for those
    #states=["Mississippi" for i in range(len(final_df))]
    #final_df["State"]=states
    
    print(f"______________________________________________________________________________________")
    print(f"{done} Files were merged & {failed} files were not merged. (Failed)\t\tMERGING IS DONE.\n______________________________________________________________________________________\n\nMoving to preprocessing() function to preprocess the columns")


    # TODO: This part can be put into a function
    try:
        # Using label encoder to encode the values of the given colummns
        le_rp_enty_type = LabelEncoder()
        final_df['reporting_entity_type'] = le_rp_enty_type.fit_transform(final_df['reporting_entity_type'])
        le_ng_type = LabelEncoder()
        final_df['negotiated_type'] = le_ng_type.fit_transform(final_df['negotiated_type'])
        print(f"Using Label Encoding to encode the values of input_file['reporting_entity_type'] & input_file['negotiated_type']")
        print("Done With the link, merged all csvs and used label encoder to encode the above columns")
        # Saving the final file
        final_df.to_csv(output_file, sep=';', index=False)
        print(f"Done with preprocessing the columns and the file is saved...\n Output File --> {output_file}")
    except:
        print("Encountered Error during encoding. Saving the file and exiting.\nCheck this file manually")
        # Saving the final file
        final_df.to_csv(output_file, sep=';', index=False)



if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Takes a dir and merges all the csvs on defined columns and saves it after preprocessing.")
    parser.add_argument('--file_path', type=str, default="./")
    parser.add_argument('--output_file', type=str, default='merged_all_csvs.csv')
    config= parser.parse_args()

    get_all_csvs(config)
