import pandas as pd
import json
import os

def convert_json_to_csv(file):
    print(f"\n===================================================\nConverting the file\n{file}")
    # Opening with pandas to check the json file
    # Opening with pandas to check the json file
    json_file = pd.read_json(file,orient='index')
    new_json_file = pd.DataFrame()
    json_file = json_file.T
    in_network=json_file['in_network']

    # FINAL columns with correct naming and valus
    negotiation_arrangements=[]
    names=[]
    billing_code_types=[]
    billing_code_type_versions=[]
    billing_codes=[]
    descriptions=[]
    negotiated_rates=[]
    expiration_dates=[]
    negotiated_types=[]
    billing_classes=[]
    reporting_entity_names=[]
    reporting_entity_types=[]
    last_updated_ons=[]
    versions=[]

    for i in range(len(in_network[0])):
        reporting_entity_names.append(json_file['reporting_entity_name'])
        reporting_entity_types.append(json_file['reporting_entity_type'])
        last_updated_ons.append(json_file['last_updated_on'])
        versions.append(json_file['version'])
        
        data=in_network[0][i]
        negotiation_arrangements.append(data['negotiation_arrangement'])
        names.append(data['name'])
        billing_code_types.append(data['billing_code_type'])
        billing_code_type_versions.append(data['billing_code_type_version'])
        billing_codes.append(data['billing_code'])
        descriptions.append(data['description'])
        # for the nested json columns
        nested_coln=data['negotiated_rates']
        dict_coln=nested_coln[0]
        negotiated_rates.append(dict_coln['negotiated_prices'][0]['negotiated_rate'])
        expiration_dates.append(dict_coln['negotiated_prices'][0]['expiration_date'])
        negotiated_types.append(dict_coln['negotiated_prices'][0]['negotiated_type'])
        billing_classes.append(dict_coln['negotiated_prices'][0]['billing_class'])

    # Appending the correct values
    new_json_file['reporting_entity_name']=reporting_entity_names
    new_json_file['reporting_entity_type']=reporting_entity_types
    new_json_file['last_updated_on']=last_updated_ons
    new_json_file['version']=versions
    new_json_file['negotiation_arrangement']=negotiation_arrangements
    new_json_file['name']=names
    new_json_file['billing_code_types']=billing_code_types
    new_json_file['billing_code_type_version']=billing_code_type_versions
    new_json_file['billing_code']=billing_codes
    new_json_file['description']=descriptions

    # Saving to csv
    #.to_csv(")
    new_json_file.to_csv("csv/"+file[:-4]+'csv',sep=';',index=False)
    print("Done with convert_json_to_csv()\n+++++++++++++++++++++++++++++++++++++++++++++++++++++++\n")


if __name__ == "__main__":
    print(f"Run from command line as:\n"
          f"python3 json_to_csv.py")
    for file in os.listdir("./"):
        file_path=os.path.join("./",file)
        file_size=os.path.getsize(file_path)
        if file.endswith('.json') and file_size<=3e8:
            try:
                convert_json_to_csv(file)
                print(f"Conversion Done...\n DELETING the .json file\n{file} DELETED..!")
                os.remove(file)
            except:
                print("Fail to convert the .json file; Not deleted. Check for manual conversion")
        else:
            print(f"File is not a _in-network.json --> {file} or its size is too big {file_size/1e6} MBs")
            if file.endswith('csv') or file.endswith(".py") or file.endswith(".json") or file.endswith(".ipynb"):
                print(f"NOT DELETED => {file}\n")                
            else:
                os.remove(file)
                print(f"File is not convertible\nDELETEING this file \n{file} DELETED..!")
